@echo OFF

:init
set CR=0

:defaults
set PROJECT=catalogCC2015.1
set WHAT2BACKUP=L:\catalogCC2015.1
set BACKUP_FOLDER=S:\Pictures\Lightroom\ramdisk-backup
set rotation=50
set UPDATE_EXCLUDE=

set ROLLING_VERSIONS_FILE=lightroomCC2015.1.lrcat
set ROLLING_VERSIONS_FILE_EXCLUDE=-xr!Backups -xr!"catalog Previews.lrdata" -xr!"catalog Smart Previews.lrdata" -xr!"catalog-ssd Previews.lrdata" -xr!"Lightroom Settings" -xr!"lightroomCC2015.1 Previews.lrdata" -xr!"lightroomCC2015.1 Smart Previews.lrdata"

::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:main
call :detect_sevenZip
call :setup_time
call :local_backup
goto :end
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:local_backup
echo %HIGH%%b%  %~0 %END%

md %BACKUP_FOLDER% 2>NUL
del /f /q "%BACKUP_FOLDER%\%PROJECT%.7z.tmp*" 2>NUL
echo testing %BACKUP_FOLDER%\%PROJECT%.7z ...
IF EXIST "%BACKUP_FOLDER%\%PROJECT%.7z" %sevenZip% t "%BACKUP_FOLDER%\%PROJECT%.7z" -bse1 2>NUL | findstr ERROR: >NUL && del /f /q "%BACKUP_FOLDER%\%PROJECT%.7z"

:: every nth rotation differential backups, we reset the master backup to decrease its size
for /f %%n in ('dir /b "%BACKUP_FOLDER%\*.7z" 2^>NUL') DO set /A _num=1
set /a "_num%%=rotation"
IF %_num% EQU 0 del /f /q "%BACKUP_FOLDER%\%PROJECT%.7z" 2>NUL

REM -slp : set Large Pages mode
REM -ms=off : disable solid
REM -bso0 : standard output messages to NUL
REM -bsp2 : progress information to stderr
REM -up0 : File exists in archive, but is not matched with wildcard = Ignore file (don't create item in new archive for this file)
REM -uq0 : File exists in archive, but deleted on disk              = Ignore file
REM -uq3 : File exists in archive, but deleted on disk              = Create Anti-item
IF EXIST "%BACKUP_FOLDER%\%PROJECT%.7z" (
  call :local_backup_update_separate_file
  call :local_backup_update
) ELSE (
  echo %c%%sevenZip% a "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -slp -bso0 -bsp2 %WHAT2BACKUP% %HIGH%%k%
  %sevenZip% a "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -slp -bso0 -bsp2 %WHAT2BACKUP%
  echo:%END%
)

goto :EOF


:: :local_backup_update_separate_file will historize non-excluded files in ROLLING_VERSIONS_FILE_EXCLUDE from the main backup separately
:: this will effefctively historize the specific files you want into separate historization zips
:: it works a bit in reverse of what you would think but that's the only way
:local_backup_update_separate_file
echo %HIGH%%b%  %~0 %END%
IF NOT DEFINED ROLLING_VERSIONS_FILE exit /b 0

:: creates a new archive %PROJECT%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z and writes to this archive all files from current directory which differ from files in %PROJECT%.7z archive
:: %PROJECT%.7z archive will also be updated after.

echo %c%%sevenZip% u "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -bso0 -bsp2 -up0q3x2z0!"%BACKUP_FOLDER%\%PROJECT%-%ROLLING_VERSIONS_FILE%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z" %WHAT2BACKUP% %ROLLING_VERSIONS_FILE_EXCLUDE%
%sevenZip% u "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -bso0 -bsp2 -up0q3x2z0!"%BACKUP_FOLDER%\%PROJECT%-%ROLLING_VERSIONS_FILE%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z" %WHAT2BACKUP% %ROLLING_VERSIONS_FILE_EXCLUDE%
echo:%END%
%sevenZip% t "%BACKUP_FOLDER%\%PROJECT%-%ROLLING_VERSIONS_FILE%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z" | findstr /C:"No files" && del /f /q "%BACKUP_FOLDER%\%PROJECT%-%ROLLING_VERSIONS_FILE%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z"

goto :EOF


:local_backup_update
echo %HIGH%%b%  %~0 %END%

:: creates a new archive %PROJECT%-%CURRENT_DATE_NOSEP%-%CURRENT_TIME%.7z and writes to this archive all files from current directory which differ from files in %PROJECT%.7z archive
:: %PROJECT%.7z archive will also be updated after.

echo %c%%sevenZip% u "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -bso0 -bsp2 -uq3 %WHAT2BACKUP% %UPDATE_EXCLUDE%
%sevenZip% u "%BACKUP_FOLDER%\%PROJECT%.7z" -ms=off -bso0 -bsp2 -uq3 %WHAT2BACKUP% %UPDATE_EXCLUDE%
echo:%END%

goto :EOF


:setup_time
IF DEFINED DEBUG echo DEBUG: %m%%~n0 %~0 %HIGH%%*%END% 1>&2

set CURRENT_DATE=%DATE:/=-%
set GOOD_DATE=%CURRENT_DATE:~0,10%
:: ISO
if "%CURRENT_DATE:~2,1%" EQU "-" set GOOD_DATE=%DATE:~6,4%-%DATE:~3,2%-%DATE:~0,2%
:: US
if "%CURRENT_DATE:~3,1%" EQU " " set GOOD_DATE=%DATE:~10,4%-%DATE:~4,2%-%DATE:~7,2%
set CURRENT_DATE_NOSEP=%GOOD_DATE:-=%

set CURRENT_TIME=%TIME::=%

:: BUG: cannot add numbers that start with 0
set /A nextHour=%CURRENT_TIME:~0,2% + 1
set nextHour=%nextHour%%TIME:~3,2%

if "%CURRENT_TIME:~0,1%" == " " set CURRENT_TIME=0%CURRENT_TIME:~1%
set CURRENT_TIME=%CURRENT_TIME:~0,4%
goto :EOF


:set_colors
set END=[0m
set HIGH=[1m
set Underline=[4m
set REVERSE=[7m

REM echo [101;93m NORMAL FOREGROUND COLORS [0m
set k=[30m
set r=[31m
set g=[32m
set y=[33m
set b=[34m
set m=[35m
set c=[36m
set w=[37m

goto :EOF


:detect_sevenZip
IF DEFINED sevenZip exit /b 0
:: adapt to your environment
for %%x in (7z.exe) DO set "sevenZip=%%~$PATH:x"
IF NOT DEFINED sevenZip IF EXIST .\7z.exe set "sevenZip=.\7z.exe"
IF NOT DEFINED sevenZip IF EXIST "%ProgramFiles%\7-Zip\7z.exe" set "sevenZip=%ProgramFiles%\7-Zip\7z.exe"
IF NOT DEFINED sevenZip IF EXIST "%ProgramFiles(x86)%\7-zip\7z.exe" set "sevenZip=%ProgramFiles(x86)%\7-zip\7z.exe"
IF NOT DEFINED sevenZip echo sevenZip NOT FOUND & timeout /t 5 & exit
goto :EOF


:end
exit /b %CR%
